const { S3Client, PutObjectCommand } = require('@aws-sdk/client-s3');
const { CloudFrontClient, CreateInvalidationCommand } = require('@aws-sdk/client-cloudfront');
const { version } = require('./package.json');
const dotenv = require('dotenv');
const fs = require('fs');
const path = require('path');

dotenv.config();

const filePath = path.join('lib', 'browser', 'index.js');
const isBeta = version.includes('beta');

const bucketName = process.env.S3_BUCKET_NAME;
const cloudfrontDistributionId = process.env.CLOUDFRONT_DISTRIBUTION_ID;
const domain = process.env.DOMAIN_NAME;

const s3Path = 'sdk/';
const versionedFile = `${s3Path}sdk-core@${version}.js`;
const latestFile = `${s3Path}sdk-core.js`;

const awsCredentials = {
	region: process.env.AWS_REGION,
	credentials: {
		accessKeyId: process.env.S3_ACCESS_KEY_ID,
		secretAccessKey: process.env.S3_SECRET_ACCESS_KEY,
	},
};

const s3 = new S3Client(awsCredentials);
const cloudfront = new CloudFrontClient(awsCredentials);

/**
 * Uploads a file to an Amazon S3 bucket.
 * @param {string} fileName - The name of the file to be uploaded to the S3 bucket.
 * @returns {Promise<void>} A promise that resolves when the file is successfully uploaded.
 */
async function uploadToS3(fileName) {
	try {
		const fileContent = fs.readFileSync(filePath);

		const params = {
			Bucket: bucketName,
			Key: fileName,
			Body: fileContent,
			ContentType: 'application/javascript',
		};

		await s3.send(new PutObjectCommand(params));
		console.log(`Archivo subido con éxito: https://${bucketName}.s3.amazonaws.com/${fileName}`);
	} catch (error) {
		console.error(`Error al subir el archivo (${fileName}):`, error);
		process.exit(1);
	}
}

/**
 * Invalidates the CloudFront cache for a specific file.
 * @param {string} fileName - The name of the file (including its path) to invalidate in the CloudFront cache.
 * @returns {Promise<void>} A promise that resolves when the cache invalidation is successfully completed.
 */
async function invalidateCloudFrontCache(fileName) {
	try {
		const params = {
			DistributionId: cloudfrontDistributionId,
			InvalidationBatch: {
				Paths: {
					Quantity: 1,
					Items: [`/${fileName}`],
				},
				CallerReference: `${Date.now()}`,
			},
		};

		await cloudfront.send(new CreateInvalidationCommand(params));
		console.log(`Caché invalidado en CloudFront para /${fileName}`);
	} catch (error) {
		console.error('Error al invalidar caché:', error);
	}
}

(async () => {
	await uploadToS3(versionedFile);
	console.log(`Archivo disponible en: ${domain}/${versionedFile}`);

	if (!isBeta) {
		await uploadToS3(latestFile);
		await invalidateCloudFrontCache(latestFile);
		console.log(`Archivo disponible en: ${domain}/${latestFile}`);
	}

	console.log('Publicación completada');
})();
