"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const crypto_js_1 = require("crypto-js");
exports.default = {
    /**
     * Serialize object to JSON string without empties
     *
     * @param value
     * @return
     */
    objectToJson(value) {
        const filtered = Object.assign({}, value);
        for (var propName in filtered) {
            if (filtered[propName] === null || filtered[propName] === undefined) {
                delete filtered[propName];
            }
        }
        return JSON.stringify(filtered);
    },
    /**
     * Helper to hash object by algorithm
     *
     * @param algorithm
     * @param value
     * @return
     */
    hash(algorithm, value) {
        switch (algorithm) {
            case 'MD5':
                return (0, crypto_js_1.MD5)(value).toString();
            case 'SHA-512':
                return (0, crypto_js_1.SHA512)(value).toString();
            default:
                return '';
        }
    },
    /**
     * Trim a string/null value
     *
     * @param value
     * @return
     */
    trimValue(value) {
        if (value) {
            return value.trim();
        }
        return null;
    },
    /**
     * Clean an string of whitespaces.
     *
     * @param value
     * @returns string | null
     */
    cleanString(value) {
        if (value) {
            return value.replace(/\s/g, '');
        }
        return null;
    },
    /**
     * Parse or nullify amount data
     *
     * @param amount
     * @return
     */
    parseAmount(amount) {
        if (typeof amount == 'number' && amount > 0) {
            return String(amount);
        }
        return null;
    },
    /**
     * Verify if environment is a Browser
     */
    checkIsBrowser() {
        return typeof window !== 'undefined' && typeof window.document !== 'undefined';
    },
    /**
     * Append website script and load
     *
     * @param src
     */
    addScriptToWebsite(src) {
        return new Promise((resolve, reject) => {
            const id = (0, crypto_js_1.MD5)(src).toString();
            const script = document.createElement('script');
            if (!document.querySelector(`script[id="${id}"]`)) {
                script.id = id;
                script.src = src;
                script.type = 'text/javascript';
                script.onload = resolve;
                script.onerror = reject;
                document.body.appendChild(script);
            }
            else {
                resolve(true);
            }
        });
    },
    /**
     * Append website style
     *
     * @param id
     * @param css
     */
    addStylesById(id, css) {
        const el = document.getElementById(`styles-${id}`);
        if (el) {
            return;
        }
        const wrap = document.createElement('style');
        wrap.id = `styles-${id}`;
        wrap.innerHTML = css;
        document.head.appendChild(wrap);
    },
    /**
     * Append secure enrollment form to site
     *
     * @param jwt
     * @param url
     */
    addSecureForm(jwt, url) {
        const iframe = document.createElement('iframe');
        iframe.id = 'cardinal_collection_iframe';
        iframe.name = 'collectionIframe';
        iframe.height = '10';
        iframe.width = '10';
        iframe.style.display = 'none';
        document.body.appendChild(iframe);
        const form = document.createElement('form');
        form.id = 'cardinal_collection_form';
        form.method = 'POST';
        form.target = 'collectionIframe';
        form.action = url;
        const input = document.createElement('input');
        input.type = 'hidden';
        input.name = 'JWT';
        input.value = jwt;
        form.appendChild(input);
        document.body.appendChild(form);
        return form;
    },
    /**
     * Remove HTML Element by ID if it exists.
     *
     * @param id
     */
    removeElementById(id) {
        const element = document.getElementById(id);
        if (element) {
            element.remove();
        }
    },
    /**
     * Convert Window Size to Dimensions
     */
    convertWindowSizeToDimensions(windowSize) {
        const dimensions = {
            '01': { width: '250', height: '400' },
            '02': { width: '390', height: '400' },
            '03': { width: '500', height: '600' },
            '04': { width: '600', height: '400' },
            '05': { width: '100%', height: '100%' },
        };
        return dimensions[windowSize || '02'] || { width: '390', height: '400' };
    },
};
//# sourceMappingURL=Helpers.js.map