"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const axios_1 = __importDefault(require("axios"));
const InvalidCredentialsException_1 = __importDefault(require("../exceptions/InvalidCredentialsException"));
const Response_1 = __importDefault(require("./Response"));
const ErrorResponse_1 = __importDefault(require("../responses/ErrorResponse"));
const FailureResponse_1 = __importDefault(require("../responses/FailureResponse"));
const InputErrorResponse_1 = __importDefault(require("../responses/InputErrorResponse"));
const NetworkFailureResponse_1 = __importDefault(require("../responses/NetworkFailureResponse"));
const NoAccessResponse_1 = __importDefault(require("../responses/NoAccessResponse"));
const NotFoundResponse_1 = __importDefault(require("../responses/NotFoundResponse"));
const PayloadResponse_1 = __importDefault(require("../responses/PayloadResponse"));
const PaymentDeclinedResponse_1 = __importDefault(require("../responses/PaymentDeclinedResponse"));
const PreconditionalResponse_1 = __importDefault(require("../responses/PreconditionalResponse"));
const SuccessResponse_1 = __importDefault(require("../responses/SuccessResponse"));
const TimeoutResponse_1 = __importDefault(require("../responses/TimeoutResponse"));
class ServiceBehaviour {
    /**
     * Initialize service
     */
    constructor(settings) {
        this.settings = settings;
    }
    /**
     * Build HTTP request to API
     *
     * @param url
     * @return
     * @throws InvalidCredentialsException
     */
    buildRequest(url, transaction) {
        if (!this.settings.auth_key || !this.settings.auth_hash) {
            throw new InvalidCredentialsException_1.default('The merchant credentials are not definied (key/hash).');
        }
        if (this.settings.environment != null) {
            transaction.env = this.settings.environment;
        }
        if (this.settings.lang != null) {
            transaction.lang = this.settings.lang;
        }
        if (this.settings.sdk != null) {
            transaction.from = this.settings.sdk;
        }
        if (this.settings.sdk_version != null) {
            transaction.sdk_version = this.settings.sdk_version;
        }
        const headers = {
            'Accept': 'application/json',
            'Content-Type': 'application/json',
            'x-auth-key': this.settings.auth_key,
            'x-auth-hash': this.settings.auth_hash,
        };
        if (this.settings.auth_user) {
            headers['x-auth-user'] = this.settings.auth_user;
        }
        return axios_1.default.create({
            baseURL: this.settings.endpoint,
            timeout: 1000 * 60,
            headers,
            validateStatus: () => true,
        });
    }
    /**
     * Mapping and cast HTTP response
     *
     * @param body
     * @param status
     * @return
     */
    parseResponse(data, status) {
        let bag = new Response_1.default();
        switch (status) {
            case 200:
                bag = new SuccessResponse_1.default();
                break;
            case 202:
                bag = new PayloadResponse_1.default();
                break;
            case 400:
                bag = new ErrorResponse_1.default();
                break;
            case 401:
            case 403:
                bag = new NoAccessResponse_1.default();
                break;
            case 402:
                bag = new PaymentDeclinedResponse_1.default();
                break;
            case 404:
            case 405:
            case 406:
                bag = new NotFoundResponse_1.default();
                break;
            case 408:
                bag = new TimeoutResponse_1.default();
                break;
            case 412:
            case 418:
                bag = new PreconditionalResponse_1.default();
                break;
            case 422:
                bag = new InputErrorResponse_1.default();
                break;
            case 500:
                bag = new FailureResponse_1.default();
                break;
            default:
                if (status > 500) {
                    bag = new NetworkFailureResponse_1.default();
                }
                break;
        }
        bag.status = status;
        bag.action = (data === null || data === void 0 ? void 0 : data.action) || null;
        bag.success = (data === null || data === void 0 ? void 0 : data.success) || false;
        bag.message = (data === null || data === void 0 ? void 0 : data.message) || null;
        bag.data = (data === null || data === void 0 ? void 0 : data.data) || null;
        bag.errors = (data === null || data === void 0 ? void 0 : data.errors) || null;
        return bag;
    }
    /**
     * Default response for exception behavior;
     *
     * @param e
     * @returns
     */
    exceptionResponse(e) {
        const response = new FailureResponse_1.default();
        response.status = 520;
        response.success = false;
        response.message = e.message;
        return response;
    }
    /**
     * Get API route
     *
     * @param route
     * @return
     */
    getRoute(route) {
        return this.settings.endpoint + '/' + route;
    }
    /**
     * API POST request
     *
     * @param url
     * @param body
     * @return
     * @throws InvalidCredentialsException
     */
    async post(url, body) {
        try {
            const response = await this.buildRequest(url, body).post(url, JSON.parse(body.toJson()));
            return this.parseResponse(response.data, response.status);
        }
        catch (e) {
            return this.exceptionResponse(e);
        }
    }
    /**
     * API PUT request
     *
     * @param url
     * @param body
     * @return
     * @throws InvalidCredentialsException
     */
    async put(url, body) {
        try {
            const response = await this.buildRequest(url, body).put(url, JSON.parse(body.toJson()));
            return this.parseResponse(response.data, response.status);
        }
        catch (e) {
            return this.exceptionResponse(e);
        }
    }
    /**
     * API DELETE request
     *
     * @param url
     * @param body
     * @return
     * @throws InvalidCredentialsException
     */
    async delete(url, body) {
        try {
            const response = await this.buildRequest(url, body).delete(url, {
                params: JSON.parse(body.toJson()),
            });
            return this.parseResponse(response.data, response.status);
        }
        catch (e) {
            return this.exceptionResponse(e);
        }
    }
    /**
     * API GET request
     *
     * @param url
     * @param body
     * @return
     * @throws InvalidCredentialsException
     */
    async get(url, body) {
        try {
            const response = await this.buildRequest(url, body).get(url, {
                params: JSON.parse(body.toJson()),
            });
            return this.parseResponse(response.data, response.status);
        }
        catch (e) {
            return this.exceptionResponse(e);
        }
    }
}
exports.default = ServiceBehaviour;
//# sourceMappingURL=ServiceBehaviour.js.map