"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Helpers_1 = __importDefault(require("../base/Helpers"));
const ServiceBehaviour_1 = __importDefault(require("../base/ServiceBehaviour"));
const InvalidTransactionTypeException_1 = __importDefault(require("../exceptions/InvalidTransactionTypeException"));
const RunningTransactionException_1 = __importDefault(require("../exceptions/RunningTransactionException"));
const PixelPayLoading_1 = __importDefault(require("../libraries/PixelPayLoading"));
const PayloadResponse_1 = __importDefault(require("../responses/PayloadResponse"));
const CardinalManager_1 = __importDefault(require("../libraries/CardinalManager"));
const PayloadManager_1 = __importDefault(require("../libraries/PayloadManager"));
const CybersourceManager_1 = __importDefault(require("../libraries/CybersourceManager"));
const RouterManager_1 = __importDefault(require("../libraries/RouterManager"));
class Transaction extends ServiceBehaviour_1.default {
    /**
     * Initialize service
     */
    constructor(settings) {
        super(settings);
    }
    /**
     * Enable concurrent transactions
     */
    static withConcurrency() {
        Transaction.concurrent = true;
    }
    /**
     * Send and proccesing SALE transaction
     *
     * @param transaction
     * @return
     * @throws InvalidCredentialsException
     */
    async doSale(transaction) {
        var _a, _b;
        if (transaction.authentication_request && !Helpers_1.default.checkIsBrowser()) {
            throw new InvalidTransactionTypeException_1.default("This platform not support 3DS transactions");
        }
        if (!Transaction.concurrent && Transaction.busy) {
            throw new RunningTransactionException_1.default("There's an active transaction still running.");
        }
        try {
            PixelPayLoading_1.default.show();
            Transaction.busy = true;
            const response = await this.post("api/v2/transaction/sale", transaction);
            if (response instanceof PayloadResponse_1.default) {
                switch ((_a = response.data) === null || _a === void 0 ? void 0 : _a.transaction_type) {
                    case 'songbird':
                        return await (new CardinalManager_1.default(this.settings, transaction)).init(response.data.payload, (_b = response.data) === null || _b === void 0 ? void 0 : _b.collection_url);
                    case 'payload':
                        return await (new PayloadManager_1.default(this.settings)).loadPayload(response.data.payload);
                    case 'router':
                        return await (new RouterManager_1.default(transaction)).init(response.data.payload);
                    default:
                        throw new RunningTransactionException_1.default("This type of transaction is invalid");
                }
            }
            CardinalManager_1.default.clearAllEvents();
            return response;
        }
        catch (err) {
            CardinalManager_1.default.clearAllEvents();
            throw err;
        }
        finally {
            PixelPayLoading_1.default.hide();
            Transaction.busy = false;
        }
    }
    /**
     * Send and proccesing AUTH transaction
     *
     * @param transaction
     * @return
     * @throws InvalidCredentialsException
     */
    async doAuth(transaction) {
        if (transaction.authentication_request && !Helpers_1.default.checkIsBrowser()) {
            throw new InvalidTransactionTypeException_1.default("This platform not support 3DS transactions");
        }
        if (!Transaction.concurrent && Transaction.busy) {
            throw new RunningTransactionException_1.default("There's an active transaction still running.");
        }
        try {
            PixelPayLoading_1.default.show();
            Transaction.busy = true;
            const response = await this.post("api/v2/transaction/auth", transaction);
            if (response instanceof PayloadResponse_1.default && response.data.transaction_type == 'songbird') {
                return await (new CardinalManager_1.default(this.settings, transaction)).init(response.data.payload, response.data.collection_url);
            }
            if (response instanceof PayloadResponse_1.default && response.data.transaction_type == 'payload') {
                return await (new PayloadManager_1.default(this.settings)).loadPayload(response.data.payload);
            }
            CardinalManager_1.default.clearAllEvents();
            return response;
        }
        catch (err) {
            CardinalManager_1.default.clearAllEvents();
            throw err;
        }
        finally {
            PixelPayLoading_1.default.hide();
            Transaction.busy = false;
        }
    }
    /**
     * Send and proccesing CAPTURE transaction
     *
     * @param transaction
     * @return
     * @throws InvalidCredentialsException
     */
    async doCapture(transaction) {
        PixelPayLoading_1.default.show();
        const response = await this.post("api/v2/transaction/capture", transaction);
        PixelPayLoading_1.default.hide();
        return response;
    }
    /**
     * Send and proccesing VOID transaction
     *
     * @param transaction
     * @return
     * @throws InvalidCredentialsException
     */
    async doVoid(transaction) {
        PixelPayLoading_1.default.show();
        const response = await this.post("api/v2/transaction/void", transaction);
        PixelPayLoading_1.default.hide();
        return response;
    }
    /**
     * Send and proccesing SALE transaction
     *
     * @param transaction
     * @return
     * @throws InvalidCredentialsException
     */
    getStatus(transaction) {
        return this.post("api/v2/transaction/status", transaction);
    }
    /**
     * Verify a payment hash and returns true if payment response is not modified
     *
     * @param hash
     * @param order_id
     * @param secret
     * @return
     */
    verifyPaymentHash(hash, order_id, secret) {
        if (Helpers_1.default.checkIsBrowser()) {
            throw new InvalidTransactionTypeException_1.default("This platform not support hash validations in browser/frontend");
        }
        const chain = [order_id, this.settings.auth_key, secret].join("|");
        return hash === Helpers_1.default.hash("MD5", chain);
    }
    /**
     * Get Cybersource Decision Manager Fingerprint
     *
     * @param merchant_id
     * @param org_id
     * @param callback
     */
    getCybersourceFingerprint(merchant_id, org_id) {
        return CybersourceManager_1.default.getFingerprint(merchant_id, org_id);
    }
}
exports.default = Transaction;
/**
 * Define if there's a transaction currently running
 */
Transaction.busy = false;
/**
 * Define if concurrent transactions should be allowed
 */
Transaction.concurrent = false;
//# sourceMappingURL=Transaction.js.map