import Billing from "../src/models/Billing";
import Card from "../src/models/Card";
import Settings from "../src/models/Settings";
import CardTokenization from "../src/requests/CardTokenization";
import ErrorResponse from "../src/responses/ErrorResponse";
import InputErrorResponse from "../src/responses/InputErrorResponse";
import SuccessResponse from "../src/responses/SuccessResponse";
import Tokenization from "../src/services/Tokenization";

const TIMEOUT = 8000;

function getCard(card_number: string = "4111 1111 1111 1111"): Card {
    var card = new Card();
    card.number = card_number;
    card.cardholder = "Test Card";
    card.expire_month = 12;
    card.expire_year = 2028;
    card.cvv2 = "999";

    return card;
}

function getBilling(): Billing {
    var billing = new Billing();
    billing.address = 'Ave Circunvalacion';
    billing.country = 'HN';
    billing.state = 'HN-CR';
    billing.city = 'San Pedro Sula';
    billing.phone = '99999999';

    return billing;
}

it('Fail vault card by missing card inputs', () => {
    expect.assertions(8)
    let settings = new Settings();
    settings.setupSandbox();

    let service: Tokenization = new Tokenization(settings);
    let cardTokenization: CardTokenization = new CardTokenization();
    cardTokenization.setBilling(getBilling());

    return service.vaultCard(cardTokenization).then( response => {
        expect(response).toBeInstanceOf(InputErrorResponse);
        expect(false).toBe(response.success);
        expect(422).toBe(response.status);
        expect(true).toBe(response.inputHasError('cvv2'))
        expect(true).toBe(response.inputHasError('expire_month'))
        expect(true).toBe(response.inputHasError('expire_year'))
        expect(true).toBe(response.inputHasError('cardholder'))
        expect(true).toBe(response.inputHasError('number'))
    })
}, TIMEOUT)

it('Fail vault card by missing billing inputs', () => {
    expect.assertions(8)
    let settings = new Settings();
    settings.setupSandbox();

    let service: Tokenization = new Tokenization(settings);
    let cardTokenization: CardTokenization = new CardTokenization();
    cardTokenization.setCard(getCard());

    return service.vaultCard(cardTokenization).then( response => {
        expect(response).toBeInstanceOf(InputErrorResponse);
        expect(false).toBe(response.success);
        expect(422).toBe(response.status);
        expect(true).toBe(response.inputHasError('address'))
        expect(true).toBe(response.inputHasError('city'))
        expect(true).toBe(response.inputHasError('country'))
        expect(true).toBe(response.inputHasError('phone'))
        expect(true).toBe(response.inputHasError('state'))
    })
}, TIMEOUT)

it('Fail vault card due invalid card', () => {
    expect.assertions(3)
    let settings = new Settings();
    settings.setupSandbox();

    let service: Tokenization = new Tokenization(settings);
    let cardTokenization: CardTokenization = new CardTokenization();
    cardTokenization.setCard(getCard('3111 2545 6985 4589'));
    cardTokenization.setBilling(getBilling());

    return service.vaultCard(cardTokenization).then( response => {
        expect(response).toBeInstanceOf(InputErrorResponse);
        expect(response.status).toBe(422);
        expect(response.message).toBe("El campo number es un número de tarjeta incorrecto.");
    })
}, TIMEOUT)

it('Success vault card', () => {
    expect.assertions(5)
    let settings = new Settings();
    settings.setupSandbox();

    let service: Tokenization = new Tokenization(settings);
    let cardTokenization: CardTokenization = new CardTokenization();
    cardTokenization.setCard(getCard());
    cardTokenization.setBilling(getBilling());

    return service.vaultCard(cardTokenization).then( response => {
        expect(response).toBeInstanceOf(SuccessResponse);
        expect(response.success).toBe(true);
        expect(response.status).toBe(200)
        expect(response.getData("mask")).toBe("411111******1111")
        let token: String = response.getData('token');
        expect(token.substring(0,2)).toBe("T-")
    })
}, TIMEOUT)

it('Success vault card with inherited signature', () => {
    expect.assertions(5)

    let settings = new Settings();
	const sdk = "sdk-ionic";
    const sdk_version = "2.0.1";

    settings.sdk = sdk;
    settings.sdk_version = sdk_version;

    settings.setupSandbox();

    let service: Tokenization = new Tokenization(settings);
    let cardTokenization: CardTokenization = new CardTokenization();
    cardTokenization.setCard(getCard());
    cardTokenization.setBilling(getBilling());

    return service.vaultCard(cardTokenization).then( response => {
        expect(response).toBeInstanceOf(SuccessResponse);
        expect(response.success).toBe(true);
        expect(response.status).toBe(200)
        expect(response.getData("mask")).toBe("411111******1111")
        let token: String = response.getData('token');
        expect(token.substring(0,2)).toBe("T-")
    })
}, TIMEOUT)

it('Fail show card info',  () => {
    expect.assertions(4)
    let settings = new Settings();
    settings.setupSandbox();

    let service: Tokenization = new Tokenization(settings);
    let cardTokenization: CardTokenization = new CardTokenization();
    cardTokenization.setCard(getCard());
    cardTokenization.setBilling(getBilling());

    return service.showCard("T-1234-1234456").then( response => {
        expect(response).toBeInstanceOf(ErrorResponse);
        expect(response.status).toBe(400)
        expect(response.success).toBe(false);
        expect(response.message).toBe("El token no es valido o tiene un formato invalid");
    });
}, TIMEOUT)

it('Success show card info', async () => {
    expect.assertions(4)
    let settings = new Settings();
    settings.setupSandbox();

    let service: Tokenization = new Tokenization(settings);
    let cardTokenization: CardTokenization = new CardTokenization();
    cardTokenization.setCard(getCard());
    cardTokenization.setBilling(getBilling());

    let response: SuccessResponse = await service.vaultCard(cardTokenization)
    let token = response.getData('token');

    return service.showCard(token).then( response => {
        expect(response).toBeInstanceOf(SuccessResponse);
        expect(response.success).toBe(true);
        expect(response.status).toBe(200)
        expect(response.getData("mask")).toBe("411111******1111")
    });
}, TIMEOUT)

it('Fail update card', () => {
    expect.assertions(4)
    let settings = new Settings();
    settings.setupSandbox();

    let service: Tokenization = new Tokenization(settings);
    let cardTokenization: CardTokenization = new CardTokenization();
    cardTokenization.setCard(getCard());
    cardTokenization.setBilling(getBilling());

    return service.updateCard('T-1234-123456-1234-123456', cardTokenization).then( response => {
        expect(response).toBeInstanceOf(ErrorResponse);
        expect(response.success).toBe(false);
        expect(response.status).toBe(400)
        expect(response.message).toBe("El token no es valido o tiene un formato invalid");
    });
}, TIMEOUT)

it('Success update card', async () => {
    expect.assertions(4)
    let settings = new Settings();
    settings.setupSandbox();

    let service: Tokenization = new Tokenization(settings);

	let cardTokenization: CardTokenization = new CardTokenization();
    cardTokenization.setCard(getCard());
    cardTokenization.setBilling(getBilling());

    let response: SuccessResponse = await service.vaultCard(cardTokenization)
    let token = response.getData('token');

	let updateCard: Card = new Card();
	updateCard.number = '5416454316599121';
    cardTokenization.setCard(updateCard);

	return service.updateCard(token, cardTokenization).then( response => {
        expect(response).toBeInstanceOf(SuccessResponse);
        expect(response.success).toBe(true);
        expect(response.status).toBe(200)
        expect(response.getData("mask")).toBe("541645******9121");
    });
}, TIMEOUT)

it('Fail delete card', () => {
    expect.assertions(4)
    let settings = new Settings();
    settings.setupSandbox();

    let service: Tokenization = new Tokenization(settings);
    let cardTokenization: CardTokenization = new CardTokenization();
    cardTokenization.setCard(getCard());
    cardTokenization.setBilling(getBilling());

    return service.deleteCard("T-1234-123456-1234-123456").then( response => {
        expect(response).toBeInstanceOf(ErrorResponse);
        expect(response.success).toBe(false);
        expect(response.status).toBe(400)
        expect(response.message).toBe("El token no es valido o tiene un formato invalid");
    });
}, TIMEOUT)

it('Success delete card', async () => {
    expect.assertions(4)
    let settings = new Settings();
    settings.setupSandbox();

    let service: Tokenization = new Tokenization(settings);
    let cardTokenization: CardTokenization = new CardTokenization();
    cardTokenization.setCard(getCard());
    cardTokenization.setBilling(getBilling());

    let response: SuccessResponse = await service.vaultCard(cardTokenization)
    let token = response.getData('token');

    return service.deleteCard(token).then( response => {
        expect(response).toBeInstanceOf(SuccessResponse);
        expect(response.status).toBe(200)
        expect(response.getData('deleted')).toBe(true);
        expect(response.message).toBe("Información obtenida con exito");
    });
}, TIMEOUT)
