import SaleTransaction from "../src/requests/SaleTransaction";
import VoidTransaction from "../src/requests/VoidTransaction";
import Settings from "../src/models/Settings"
import Transaction from "../src/services/Transaction";
import FailureResponse from "../src/responses/FailureResponse";
import Card from "../src/models/Card";
import Billing from "../src/models/Billing";
import Order from "../src/models/Order";
import SuccessResponse from "../src/responses/SuccessResponse";
import InputErrorResponse from "../src/responses/InputErrorResponse";
import PaymentDeclinedResponse from "../src/responses/PaymentDeclinedResponse";
import ErrorResponse from "../src/responses/ErrorResponse";
import NotFoundResponse from "../src/responses/NotFoundResponse";
import PreconditionalResponse from "../src/responses/PreconditionalResponse";
import TimeoutResponse from "../src/responses/TimeoutResponse";
import Helpers from "../src/base/Helpers";

const TIMEOUT = 30000;

function getCard(card_number: string = "4111 1111 1111 1111") {
    var card = new Card();
    card.number = card_number;
    card.cardholder = "Test Card";
    card.expire_month = 12;
    card.expire_year = 2030;
    card.cvv2 = "999";

    return card;
}

function getBilling() {
    var billing = new Billing();
    billing.address = 'Ave Circunvalacion';
    billing.country = 'HN';
    billing.state = 'HN-CR';
    billing.city = 'San Pedro Sula';
    billing.phone = '99999999';

    return billing;
}

function getOrder(amount?: number) {
    var order = new Order();
    order.id = 'TEST-1234';
    order.currency = 'HNL';
    order.amount = amount ?? 1.00;
    order.customer_name = 'Jhon Doe';
    order.customer_email = 'jhondow@pixel.hn';

    return order;
}

function getSaleService(): Transaction {
    let settings = new Settings();
    settings.setupSandbox();
    let service: Transaction = new Transaction(settings);

    return service;
}

function getSaleRequest(amount?: number): SaleTransaction {
    let sale = new SaleTransaction();
    sale.setBilling(getBilling());
    sale.setCard(getCard());
    sale.setOrder(getOrder(amount));

    return sale;
}

it('Test invalid credentials', () => {
    expect.assertions(1);
    let settings = new Settings();
    let service = new Transaction(settings)

    let sale = new SaleTransaction();
    return service.doSale(sale).then( response => {
        expect(response).toBeInstanceOf(FailureResponse);
    }).catch( error => {
        expect(FailureResponse).toBeInstanceOf(error);
        expect('The merchant credentials are not definied (key/hash).').toBe(error.message)
    });
}, TIMEOUT)

it('Transaction sale by invalid card fields', () => {
    expect.assertions(7);
    let settings = new Settings();
    let service = new Transaction(settings)
    settings.setupSandbox();

    let sale = new SaleTransaction();
    sale.setBilling(getBilling());
    sale.setOrder(getOrder());

    return service.doSale(sale).then( response => {
        expect(response).toBeInstanceOf(InputErrorResponse);
        expect(false).toBe(response.success);
        expect(422).toBe(response.status);
        expect(true).toBe(response.inputHasError('card_cvv'))
        expect(true).toBe(response.inputHasError('card_expire'))
        expect(true).toBe(response.inputHasError('card_holder'))
        expect(true).toBe(response.inputHasError('card_number'))
    })
}, TIMEOUT)

it('Transaction sale by invalid billing fields', () => {
    expect.assertions(8);
    let settings = new Settings();
    let service = new Transaction(settings)
    settings.setupSandbox();

    let sale = new SaleTransaction();
    sale.setCard(getCard());
    sale.setOrder(getOrder());

    return service.doSale(sale).then( response => {
        expect(response).toBeInstanceOf(InputErrorResponse);
        expect(false).toBe(response.success);
        expect(422).toBe(response.status);
        expect(true).toBe(response.inputHasError('billing_address'))
        expect(true).toBe(response.inputHasError('billing_city'))
        expect(true).toBe(response.inputHasError('billing_country'))
        expect(true).toBe(response.inputHasError('billing_phone'))
        expect(true).toBe(response.inputHasError('billing_state'))
    })
}, TIMEOUT)

it('Transaction sale by invalid order fields', () => {
    expect.assertions(8);
    let settings = new Settings();
    let service = new Transaction(settings)
    settings.setupSandbox();

    let sale = new SaleTransaction();
    sale.setCard(getCard());
    sale.setBilling(getBilling());

    return service.doSale(sale).then( response => {
        expect(response).toBeInstanceOf(InputErrorResponse);
        expect(false).toBe(response.success);
        expect(422).toBe(response.status);
        expect(true).toBe(response.inputHasError('customer_email'));
        expect(true).toBe(response.inputHasError('customer_name'));
        expect(true).toBe(response.inputHasError('order_amount'));
        expect(true).toBe(response.inputHasError('order_currency'));
        expect(true).toBe(response.inputHasError('order_id'));
    })
}, TIMEOUT)

it('Test successful transaction', () => {
    expect.assertions(1);
    let service: Transaction = getSaleService();
    let sale: SaleTransaction = getSaleRequest();

    return service.doSale(sale).then( response => {
        expect(response).toBeInstanceOf(SuccessResponse);
    })
}, TIMEOUT)

it('test declined sale transaction', () => {
    expect.assertions(3)
    let service: Transaction = getSaleService();
    let sale: SaleTransaction = getSaleRequest(2);

    return service.doSale(sale).then( response => {
        expect(response).toBeInstanceOf(PaymentDeclinedResponse);
        expect(response.message).toBe("Transacción denegada")
        expect(402).toBe(response.status)
    })
}, TIMEOUT)

it('testInvalidSetupMerchantSettings', () => {
    expect.assertions(3)
    let service: Transaction = getSaleService();
    let sale: SaleTransaction = getSaleRequest(3);

    return service.doSale(sale).then( response => {
        expect(response).toBeInstanceOf(ErrorResponse);
        expect(response.message).toBe("Configuración de comercio inválida");
        expect(400).toBe(response.status)
    })
}, TIMEOUT)


it('test failed transaction due card report', () => {
    expect.assertions(3)
    let service: Transaction = getSaleService();
    let sale: SaleTransaction = getSaleRequest(4);

    return service.doSale(sale).then( response => {
        expect(response).toBeInstanceOf(PaymentDeclinedResponse);
        expect(response.message).toBe("Tarjeta con reporte de robo o extravío");
        expect(402).toBe(response.status)
    })
}, TIMEOUT)

it('test payment not fount', () => {
    expect.assertions(3)
    let service: Transaction = getSaleService();
    let sale: SaleTransaction = getSaleRequest(5);

    return service.doSale(sale).then( response => {
        expect(response).toBeInstanceOf(NotFoundResponse);
        expect(response.message).toBe("Error al encontrar el cobro");
        expect(406).toBe(response.status)
    })
}, TIMEOUT)

it('test fail by limits exceeded', () => {
    expect.assertions(3)
    let service: Transaction = getSaleService();
    let sale: SaleTransaction = getSaleRequest(6);

    return service.doSale(sale).then( response => {
        expect(response).toBeInstanceOf(PreconditionalResponse);
        expect(response.message).toBe("El límite de intentos para esta tarjeta ha sido superado");
        expect(412).toBe(response.status)
    })
}, TIMEOUT)

it('test general system failure', () => {
    expect.assertions(3)
    let service: Transaction = getSaleService();
    let sale: SaleTransaction = getSaleRequest(7);

    return service.doSale(sale).then( response => {
        expect(response).toBeInstanceOf(FailureResponse);
        expect(response.message).toBe("Error general del sistema");
        expect(500).toBe(response.status)
    })
}, TIMEOUT)

it('test timeout error', () => {
    expect.assertions(3)
    let service: Transaction = getSaleService();
    let sale: SaleTransaction = getSaleRequest(8);

    return service.doSale(sale).then( response => {
        expect(response).toBeInstanceOf(TimeoutResponse);
        expect(response.message).toBe("Tiempo de espera excedido");
        expect(408).toBe(response.status)
    })
}, TIMEOUT)


it('test transaction amount exceeded', () => {
    expect.assertions(3)
    let service: Transaction = getSaleService();
    let sale: SaleTransaction = getSaleRequest(9);

    return service.doSale(sale).then( response => {
        expect(response).toBeInstanceOf(PreconditionalResponse);
        expect(response.message).toBe("Lo sentimos, este valor excede el máximo permitido al comercio");
        expect(412).toBe(response.status)
    })
}, TIMEOUT)

it('test transaction limit exceeded', () => {
    expect.assertions(3)
    let service: Transaction = getSaleService();
    let sale: SaleTransaction = getSaleRequest(10);

    return service.doSale(sale).then( response => {
        expect(response).toBeInstanceOf(PreconditionalResponse);
        expect(response.message).toBe("Lo sentimos, este comercio ha llegado a su límite de transacciones");
        expect(412).toBe(response.status)
    })
}, TIMEOUT)

it('test ConfigurationLimits', () => {
    expect.assertions(3)
    let service: Transaction = getSaleService();
    let sale: SaleTransaction = getSaleRequest(11);

    return service.doSale(sale).then( response => {
        expect(response).toBeInstanceOf(PreconditionalResponse);
        expect(response.message).toBe("Lo sentimos, este comercio ha llegado a su límite de ventas");
        expect(412).toBe(response.status)
    })
}, TIMEOUT)

it('transaction fail by not accepted cards', () => {
    expect.assertions(3)
    let service: Transaction = getSaleService();
    let sale: SaleTransaction = getSaleRequest(12);

    return service.doSale(sale).then( response => {
        expect(response).toBeInstanceOf(PreconditionalResponse);
        expect(response.message).toBe("El comercio no acepta tarjetas de este emisor.");
        expect(412).toBe(response.status)
    })
}, TIMEOUT)

it('Transaction fail due not accepted card type', () => {
    expect.assertions(3)
    let service: Transaction = getSaleService();
    let sale: SaleTransaction = getSaleRequest(13);

    return service.doSale(sale).then( response => {
        expect(response).toBeInstanceOf(PreconditionalResponse);
        expect(response.message).toBe("El comercio no acepta tarjetas de este tipo.");
        expect(412).toBe(response.status)
    })
}, TIMEOUT)

it('Transaction fail due not accepted payments for a country or region', () => {
    expect.assertions(3)
    let service: Transaction = getSaleService();
    let sale: SaleTransaction = getSaleRequest(14);

    return service.doSale(sale).then( response => {
        expect(response).toBeInstanceOf(PreconditionalResponse);
        expect(response.message).toBe("El comercio no permite transacciones de este país o región.");
        expect(412).toBe(response.status)
    })
}, TIMEOUT)

it('Transaction with Installment', () => {
    let sale: SaleTransaction = new SaleTransaction();

	sale.setInstallment(12, 'extra');

	expect(sale.installment_months).toBe('12');
	expect(sale.installment_type).toBe('extra');
})

it('Void Transaction with signature', () => {
	let voidTx: VoidTransaction = new VoidTransaction();

	voidTx.void_signature = "d206a1e51ffd42f0a8c4146d4a06846f560df211e28ec011e92b8dd4648d454f64be3ca50f753fb624f8c4c3585b3ebb6b3a954869818728f23a78b474a5b1fc";

	expect(voidTx.void_signature).toBe(Helpers.hash('SHA-512', "sonia@pixel.hn|20230530|@s4ndb0x-abcd-1234-n1l4-p1x3l"));
})
